from . import logger
from . import rinexfile
from . import metadata
from . import gamit_meta
from . import rinexmod_api
from . import get_m3g

__version__='3.6.3' #  changed automaticcaly with bump-my-version

